@@ -7,56 +7,59 @@
 #include "config.h"
 #endif /* HAVE_CONFIG_H */
 
+#include <arpa/inet.h>
+#include <ctype.h>
+#include <errno.h>
+#include <grp.h>
+#include <inttypes.h>
+#include <netinet/in.h>
+#include <pwd.h>
 #include <stdio.h>
 #include <stdlib.h>
-#include <ctype.h>
-#include <pwd.h>
-#include <grp.h>
-#include <utime.h>
-#include <errno.h>
 #include <string.h>
 #include <sys/param.h>
 #include <sys/socket.h>
-#include <netinet/in.h>
-#include <arpa/inet.h>
-#include <inttypes.h>
 #include <time.h>
+#include <utime.h>
 
-#include <atalk/dsi.h>
 #include <atalk/adouble.h>
 #include <atalk/afp.h>
-#include <atalk/util.h>
-#include <atalk/logger.h>
-#include <atalk/vfs.h>
-#include <atalk/uuid.h>
-#include <atalk/ea.h>
-#include <atalk/bstrlib.h>
 #include <atalk/bstradd.h>
-#include <atalk/ftw.h>
-#include <atalk/globals.h>
-#include <atalk/fce_api.h>
-#include <atalk/errchk.h>
-#include <atalk/iniparser.h>
-#include <atalk/unix.h>
-#include <atalk/netatalk_conf.h>
-#include <atalk/server_ipc.h>
-
-#ifdef HAVE_LDAP
-#include <atalk/ldapconfig.h>
-#endif /* HAVE_LDAP */
+#include <atalk/bstrlib.h>
 
 #ifdef CNID_DB
 #include <atalk/cnid.h>
 #endif /* CNID_DB*/
 
+#include <atalk/dsi.h>
+#include <atalk/ea.h>
+#include <atalk/errchk.h>
+#include <atalk/fce_api.h>
+#include <atalk/ftw.h>
+#include <atalk/globals.h>
+#include <atalk/iniparser.h>
+
+#ifdef HAVE_LDAP
+#include <atalk/ldapconfig.h>
+#endif /* HAVE_LDAP */
+
+#include <atalk/logger.h>
+#include <atalk/netatalk_conf.h>
+#include <atalk/server_ipc.h>
+#include <atalk/unix.h>
+#include <atalk/util.h>
+#include <atalk/uuid.h>
+#include <atalk/vfs.h>
+#include <atalk/volume.h>
+
+#include "acls.h"
 #include "directory.h"
 #include "file.h"
-#include "volume.h"
-#include "unix.h"
-#include "mangle.h"
 #include "fork.h"
 #include "hash.h"
-#include "acls.h"
+#include "mangle.h"
+#include "unix.h"
+#include "volume.h"
 
 #define VOLPASSLEN  8
 
@@ -94,6 +97,8 @@ EC_CLEANUP:
     if (file)
         fclose(file);
     LOG(log_debug, logtype_afpd, "get_tm_bandsize(\"%s\"): bandsize: %lld", path, bandsize);
+    if (ret != 0)
+        return -1;
     return bandsize;
 }
 
@@ -115,7 +120,7 @@ static long long int get_tm_bands(const char *path)
     while ((entry = readdir(dir)) != NULL)
         count++;
     count -= 2; /* All OSens I'm aware of return "." and "..", so just substract them, avoiding string comparison in loop */
-        
+
 EC_CLEANUP:
     if (dir)
         closedir(dir);
@@ -180,7 +185,7 @@ static int get_tm_used(struct vol * restrict vol)
             && (strlen(entry->d_name) == (p + strlen("sparsebundle") - entry->d_name))) {
 
             EC_NULL_LOG( infoplist = bformat("%s/%s/%s", vol->v_path, entry->d_name, "Info.plist") );
-            
+
             if ((bandsize = get_tm_bandsize(cfrombstr(infoplist))) == -1) {
                 bdestroy(infoplist);
                 infoplist = NULL;
@@ -222,24 +227,17 @@ static int getvolspace(const AFPObj *obj, struct vol *vol,
                        uint32_t *bfree, uint32_t *btotal,
                        VolSpace *xbfree, VolSpace *xbtotal, uint32_t *bsize)
 {
-    int         spaceflag, rc;
+    int         rc;
     uint32_t   maxsize;
 #ifndef NO_QUOTA_SUPPORT
     VolSpace    qfree, qtotal;
+    int         spaceflag = AFPVOL_GVSMASK & vol->v_flags;
 #endif
 
-    spaceflag = AFPVOL_GVSMASK & vol->v_flags;
     /* report up to 2GB if afp version is < 2.2 (4GB if not) */
-    maxsize = (obj->afp_version < 22) ? 0x7fffffffL : 0xffffffffL;
-
-#ifdef AFS
-    if ( spaceflag == AFPVOL_NONE || spaceflag == AFPVOL_AFSGVS ) {
-        if ( afs_getvolspace( vol, xbfree, xbtotal, bsize ) == AFP_OK ) {
-            vol->v_flags = ( ~AFPVOL_GVSMASK & vol->v_flags ) | AFPVOL_AFSGVS;
-            goto getvolspace_done;
-        }
-    }
-#endif
+    maxsize = (vol->v_flags & AFPVOL_A2VOL) ? 0x01fffe00 :
+        (((obj->afp_version < 22) || (vol->v_flags & AFPVOL_LIMITSIZE))
+         ? 0x7fffffffL : 0xffffffffL);
 
     if (( rc = ustatfs_getvolspace( vol, xbfree, xbtotal, bsize)) != AFP_OK ) {
         return( rc );
@@ -301,14 +299,15 @@ static void vol_setdate(uint16_t id, struct adouble *adp, time_t date)
 static int getvolparams(const AFPObj *obj, uint16_t bitmap, struct vol *vol, struct stat *st, char *buf, size_t *buflen)
 {
     struct adouble  ad;
-    int         bit = 0, isad = 1;
-    uint32_t       aint;
-    u_short     ashort;
-    uint32_t       bfree, btotal, bsize;
-    VolSpace            xbfree, xbtotal; /* extended bytes */
-    char        *data, *nameoff = NULL;
-    char                *slash;
-    char        *ade = NULL;
+    int bit = 0, isad = 1;
+    uint32_t aint;
+    uint16_t ashort;
+    uint32_t bfree = 0, btotal = 0, bsize = 0;
+    VolSpace xbfree = 0, xbtotal = 0; /* extended bytes */
+    char *data = NULL;
+    char *nameoff = NULL;
+    const char *slash = NULL;
+    char *ade = NULL;
 
     LOG(log_debug, logtype_afpd, "getvolparams: Volume '%s'", vol->v_localname);
 
@@ -330,11 +329,8 @@ static int getvolparams(const AFPObj *obj, uint16_t bitmap, struct vol *vol, str
             slash++;
         else
             slash = vol->v_path;
-        if (ad_getentryoff(&ad, ADEID_NAME)) {
+        if (ad_getentryoff(&ad, ADEID_NAME) && (ade = ad_entry(&ad, ADEID_NAME)) != NULL) {
             ad_setentrylen( &ad, ADEID_NAME, strlen( slash ));
-            ade = ad_entry(&ad, ADEID_NAME);
-            AFP_ASSERT(ade != NULL);
-
             memcpy(ade, slash, ad_getentrylen( &ad, ADEID_NAME ));
         }
         vol_setdate(vol->v_vid, &ad, st->st_mtime);
@@ -581,7 +577,7 @@ int afp_getsrvrparms(AFPObj *obj, char *ibuf _U_, size_t ibuflen _U_, char *rbuf
         /*
          * There seems to be an undocumented limit on how big our reply can get
          * before the client chokes and closes the connection.
-         * Testing with 10.8.4 found the limit at ~4600 bytes. Go figure. 
+         * Testing with 10.8.4 found the limit at ~4600 bytes. Go figure.
          */
         if (((data + len + 3) - rbuf) > 4600)
             break;
@@ -589,6 +585,13 @@ int afp_getsrvrparms(AFPObj *obj, char *ibuf _U_, size_t ibuflen _U_, char *rbuf
         /* set password bit if there's a volume password */
         *data = (volume->v_password) ? AFPSRVR_PASSWD : 0;
 
+        /* Apple 2 clients running ProDOS-8 expect one volume to have
+           bit 0 of this byte set.  They will not recognize anything
+           on the server unless this is the case.  I have not
+           completely worked this out, but it's related to booting
+           from the server.  Support for that function is a ways
+           off.. <shirsch@ibm.net> */
+        *data |= (volume->v_flags & AFPVOL_A2VOL) ? AFPSRVR_CONFIGINFO : 0;
         *data++ |= 0; /* UNIX PRIVS BIT ..., OSX doesn't seem to use it, so we don't either */
         *data++ = len;
         memcpy(data, namebuf, len );
@@ -647,7 +650,7 @@ static int volume_codepage(AFPObj *obj, struct vol *volume)
 }
 
 /* ------------------------- */
-static int volume_openDB(const AFPObj *obj, struct vol *volume)
+static int volume_openDB(const AFPObj *obj _U_, struct vol *volume)
 {
     int flags = 0;
 
@@ -657,23 +660,10 @@ static int volume_openDB(const AFPObj *obj, struct vol *volume)
 
     volume->v_cdb = cnid_open(volume, volume->v_cnidscheme, flags);
 
-    if ( ! volume->v_cdb && ! (flags & CNID_FLAG_MEMORY)) {
-        /* The first attempt failed and it wasn't yet an attempt to open in-memory */
+    if ( ! volume->v_cdb ) {
         LOG(log_error, logtype_afpd, "Can't open volume \"%s\" CNID backend \"%s\" ",
             volume->v_path, volume->v_cnidscheme);
-        LOG(log_error, logtype_afpd, "Reopen volume %s using in memory temporary CNID DB.",
-            volume->v_path);
-        flags |= CNID_FLAG_MEMORY;
-        volume->v_cdb = cnid_open(volume, "tdb", flags);
-#ifdef SERVERTEXT
-        /* kill ourself with SIGUSR2 aka msg pending */
-        if (volume->v_cdb) {
-            setmessage("Something wrong with the volume's CNID DB, using temporary CNID DB instead."
-                       "Check server messages for details!");
-            kill(getpid(), SIGUSR2);
-            /* deactivate cnid caching/storing in AppleDouble files */
-        }
-#endif
+        return -1;
     }
 
     return (!volume->v_cdb)?-1:0;
@@ -975,6 +965,10 @@ int  pollvoltime(AFPObj *obj)
     struct vol       *vol;
     struct timeval   tv;
     struct stat      st;
+    if (obj->proto == AFPPROTO_DSI)
+    {
+        obj->handle = obj->dsi;
+    }
 
     if (!(obj->afp_version > 21 && obj->options.flags & OPTION_SERVERNOTIF))
         return 0;
@@ -986,7 +980,7 @@ int  pollvoltime(AFPObj *obj)
         if ( (vol->v_flags & AFPVOL_OPEN)  && vol->v_mtime + 30 < tv.tv_sec) {
             if ( !stat( vol->v_path, &st ) && vol->v_mtime != st.st_mtime ) {
                 vol->v_mtime = st.st_mtime;
-                if (!obj->attention(obj->dsi, AFPATTN_NOTIFY | AFPATTN_VOLCHANGED))
+                if (!obj->attention(obj->handle, AFPATTN_NOTIFY | AFPATTN_VOLCHANGED))
                     return -1;
                 return 1;
             }
@@ -999,6 +993,10 @@ int  pollvoltime(AFPObj *obj)
 void setvoltime(AFPObj *obj, struct vol *vol)
 {
     struct timeval  tv;
+    if (obj->proto == AFPPROTO_DSI)
+    {
+        obj->handle = obj->dsi;
+    }
 
     if ( gettimeofday( &tv, NULL ) < 0 ) {
         LOG(log_error, logtype_afpd, "setvoltime(%s): gettimeofday: %s", vol->v_path, strerror(errno) );
@@ -1017,7 +1015,7 @@ void setvoltime(AFPObj *obj, struct vol *vol)
          * AFP 3.2 and above clients seem to be ok without so many notification
          */
         if (obj->afp_version < 32 && obj->options.flags & OPTION_SERVERNOTIF) {
-            obj->attention(obj->dsi, AFPATTN_NOTIFY | AFPATTN_VOLCHANGED);
+            obj->attention(obj->handle, AFPATTN_NOTIFY | AFPATTN_VOLCHANGED);
         }
     }
 }
